const std = @import("std");
const mem = std.mem;
const heap = std.heap;

const h = @import("libvirt-helper.zig");
const c = @import("libvirt-c.zig").c;
const err = @import("libvirt-error.zig");
const Connection = @import("libvirt-connection.zig");
const Domain = @import("libvirt-domain.zig");
const VirError = err.VirError;
const Snapshot = @This();

ptr: c.virDomainSnapshotPtr,
arena: *heap.ArenaAllocator,

pub fn init(ptr: c.virDomainSnapshotPtr, allocator: mem.Allocator) Snapshot {
    var arena = heap.ArenaAllocator.init(allocator);
    return .{
        .ptr = ptr,
        .arena = &arena,
    };
}
pub fn deinit(self: *const Snapshot) void {
    self.arena.deinit();
}

pub fn hasCurrentSnapshot(domain: *const Domain) VirError!bool {
    return switch (c.virDomainHasCurrentSnapshot(domain.ptr, 0)) {
        0 => false,
        1 => true,
        else => err.handleError(),
    };
}
// pub fn listAllSnapshots(domain: *const Domain) VirError!void {} // TODO
pub fn revertToSnapshot() void {}
pub fn createXML(domain: *const Domain, xml: []const u8, flags: []const CreateFlags) VirError!Snapshot {
    return if (c.virDomainSnapshotCreateXML(domain.ptr, @ptrCast(xml), h.intFromFlags(CreateFlags, flags))) |s| s else err.handleError();
}
pub fn current(domain: *const Domain) VirError!Snapshot {
    return if (c.virDomainSnapshotCurrent(domain.ptr, 0)) |s| s else err.handleError();
}

pub fn delete(self: *const Snapshot, flags: []const DeleteFlags) VirError!void {
    if (c.virDomainSnapshotDelete(self.ptr, h.intFromFlags(DeleteFlags, flags)) < 0) return err.handleError();
}
pub fn free(self: *const Snapshot) VirError!void {
    if (c.virDomainSnapshotFree(self.ptr) < 0) return err.handleError();
}
pub fn getConnect(self: *const Snapshot) VirError!Connection {
    return if (c.virDomainSnapshotGetConnect(self.ptr)) |v| v else err.handleError();
}
pub fn getDomain(self: *const Snapshot) VirError!Domain {
    return if (c.virDomainSnapshotGetDomain(self.ptr)) |v| v else err.handleError();
}
pub fn getName(self: *const Snapshot) VirError![]const u8 {
    return if (c.virDomainSnapshotGetName(self.ptr)) |v| v else err.handleError();
}
pub fn getParent(self: *const Snapshot) VirError!Snapshot {
    return if (c.virDomainSnapshotGetParent(self.ptr, 0)) |v| v else err.handleError();
}
// pub fn getXMLDesc(self: *const Snapshot) VirError!void {} // TODO
pub fn hasMetadata(self: *const Snapshot) VirError!bool {
    return switch (c.virDomainSnapshotHasMetadata(self.ptr, 0)) {
        0 => false,
        1 => true,
        else => err.handleError(),
    };
}
pub fn isCurrent(self: *const Snapshot) VirError!bool {
    return switch (c.virDomainSnapshotIsCurrent(self.ptr, 0)) {
        0 => false,
        1 => true,
        else => err.handleError(),
    };
}

pub fn listAllChildren() void {} // TODO
pub fn listChildrenNames() void {} // TODO
// pub fn listNames(domain: *const Domain) VirError!void {} // TODO

pub fn lookupByName(domain: *const Domain, name: []const u8) VirError!Snapshot {
    return if (c.virDomainSnapshotLookupByName(domain.ptr, @ptrCast(name), 0)) |s| s else err.handleError();
}
pub fn num(domain: *const Domain, flags: []const ListFlags) VirError!u32 {
    const n = c.virDomainSnapshotNum(domain.ptr, h.intFromFlags(ListFlags, flags));
    return if (n < 0) err.handleError() else @intCast(n);
}
pub fn numChildren(self: *const Snapshot, flags: []const ListFlags) VirError!u32 {
    const n = c.virDomainSnapshotNumChildred(self.ptr, h.intFromFlags(ListFlags, flags));
    return if (n < 0) err.handleError() else @intCast(n);
}
pub fn ref(self: *const Snapshot) VirError!void {
    if (c.virDomainSnapshotRef(self.ptr) < 0) return err.handleError();
}

pub const ListFlags = enum(c_uint) {
    Descendants = c.VIR_DOMAIN_SNAPSHOT_LIST_DESCENDANTS,
    Roots = c.VIR_DOMAIN_SNAPSHOT_LIST_ROOTS,
    Metadata = c.VIR_DOMAIN_SNAPSHOT_LIST_METADATA,
    Leaves = c.VIR_DOMAIN_SNAPSHOT_LIST_LEAVES,
    NoLeaves = c.VIR_DOMAIN_SNAPSHOT_LIST_NO_LEAVES,
    NoMetadata = c.VIR_DOMAIN_SNAPSHOT_LIST_NO_METADATA,
    Inactive = c.VIR_DOMAIN_SNAPSHOT_LIST_INACTIVE,
    Active = c.VIR_DOMAIN_SNAPSHOT_LIST_ACTIVE,
    DiskOnly = c.VIR_DOMAIN_SNAPSHOT_LIST_DISK_ONLY,
    Internal = c.VIR_DOMAIN_SNAPSHOT_LIST_INTERNAL,
    External = c.VIR_DOMAIN_SNAPSHOT_LIST_EXTERNAL,
    Topological = c.VIR_DOMAIN_SNAPSHOT_LIST_TOPOLOGICAL,
};
pub const CreateFlags = enum(c_uint) {
    Redefine = c.VIR_DOMAIN_SNAPSHOT_CREATE_REDEFINE,
    Current = c.VIR_DOMAIN_SNAPSHOT_CREATE_CURRENT,
    NoMetadata = c.VIR_DOMAIN_SNAPSHOT_CREATE_NO_METADATA,
    Halt = c.VIR_DOMAIN_SNAPSHOT_CREATE_HALT,
    DiskOnly = c.VIR_DOMAIN_SNAPSHOT_CREATE_DISK_ONLY,
    ReuseExt = c.VIR_DOMAIN_SNAPSHOT_CREATE_REUSE_EXT,
    Quiesce = c.VIR_DOMAIN_SNAPSHOT_CREATE_QUIESCE,
    Atomic = c.VIR_DOMAIN_SNAPSHOT_CREATE_ATOMIC,
    Live = c.VIR_DOMAIN_SNAPSHOT_CREATE_LIVE,
    Validate = c.VIR_DOMAIN_SNAPSHOT_CREATE_VALIDATE,
};
pub const DeleteFlags = enum(c_uint) {
    Children = c.VIR_DOMAIN_SNAPSHOT_DELETE_CHILDREN,
    MetadataOnly = c.VIR_DOMAIN_SNAPSHOT_DELETE_METADATA_ONLY,
    ChildrenOnly = c.VIR_DOMAIN_SNAPSHOT_DELETE_CHILDREN_ONLY,
};
pub const RevertFlags = enum(c_uint) {
    Running = c.VIR_DOMAIN_SNAPSHOT_REVERT_RUNNING,
    Paused = c.VIR_DOMAIN_SNAPSHOT_REVERT_PAUSED,
    Force = c.VIR_DOMAIN_SNAPSHOT_REVERT_FORCE,
    ResetNvram = c.VIR_DOMAIN_SNAPSHOT_REVERT_RESET_NVRAM,
};
pub const XMLFlags = enum(c_uint) {
    Secure = c.VIR_DOMAIN_XML_SECURE,
};
